@@ -12,6 +12,7 @@ from typing import (
     List,
     Optional,
     Sequence,
+    Union,
 )
 
 import yaml
@@ -76,7 +77,7 @@ class BaseSwarm(ABC):
         self,
         name: Optional[str] = None,
         description: Optional[str] = None,
-        agents: Optional[List[Agent]] = None,
+        agents: Optional[List[Union[Agent, Callable]]] = None,
         models: Optional[List[Any]] = None,
         max_loops: Optional[int] = 200,
         callbacks: Optional[Sequence[callable]] = None,
@@ -89,7 +90,6 @@ class BaseSwarm(ABC):
         stopping_function: Optional[Callable] = None,
         stopping_condition: Optional[str] = "stop",
         stopping_condition_args: Optional[Dict] = None,
-        agentops_on: Optional[bool] = False,
         speaker_selection_func: Optional[Callable] = None,
         rules: Optional[str] = None,
         collective_memory_system: Optional[Any] = False,
@@ -112,7 +112,6 @@ class BaseSwarm(ABC):
         self.stopping_function = stopping_function
         self.stopping_condition = stopping_condition
         self.stopping_condition_args = stopping_condition_args
-        self.agentops_on = agentops_on
         self.speaker_selection_func = speaker_selection_func
         self.rules = rules
         self.collective_memory_system = collective_memory_system
@@ -167,11 +166,6 @@ class BaseSwarm(ABC):
             self.stopping_condition_args = stopping_condition_args
             self.stopping_condition = stopping_condition
 
-        # If agentops is enabled, try to import agentops
-        if agentops_on is True:
-            for agent in self.agents:
-                agent.agent_ops_on = True
-
         # Handle speaker selection function
         if speaker_selection_func is not None:
             if not callable(speaker_selection_func):
