@@ -1,3 +1,4 @@
+import concurrent.futures
 import asyncio
 import os
 import threading
@@ -5,7 +6,7 @@ from concurrent.futures import (
     ThreadPoolExecutor,
 )
 from dataclasses import dataclass
-from typing import Any, List
+from typing import Any, Callable, List, Optional, Union
 
 import psutil
 
@@ -68,44 +69,42 @@ async def run_agents_concurrently_async(
 def run_agents_concurrently(
     agents: List[AgentType],
     task: str,
-    batch_size: int = None,
-    max_workers: int = None,
+    max_workers: Optional[int] = None,
 ) -> List[Any]:
     """
-    Optimized concurrent agent runner using both uvloop and ThreadPoolExecutor.
+    Optimized concurrent agent runner using ThreadPoolExecutor.
 
     Args:
         agents: List of Agent instances to run concurrently
         task: Task string to execute
-        batch_size: Number of agents to run in parallel in each batch (defaults to CPU count)
-        max_workers: Maximum number of threads in the executor (defaults to CPU count * 2)
+        max_workers: Maximum number of threads in the executor (defaults to 95% of CPU cores)
 
     Returns:
         List of outputs from each agent
     """
-    # Optimize defaults based on system resources
-    cpu_cores = os.cpu_count()
-    batch_size = batch_size or cpu_cores
-    max_workers = max_workers or cpu_cores * 2
+    if max_workers is None:
+        # 95% of the available CPU cores
+        num_cores = os.cpu_count()
+        max_workers = int(num_cores * 0.95) if num_cores else 1
 
     results = []
 
-    # Get or create event loop
-    try:
-        loop = asyncio.get_event_loop()
-    except RuntimeError:
-        loop = asyncio.new_event_loop()
-        asyncio.set_event_loop(loop)
+    with concurrent.futures.ThreadPoolExecutor(
+        max_workers=max_workers
+    ) as executor:
+        # Submit all tasks and get futures
+        futures = [
+            executor.submit(agent.run, task) for agent in agents
+        ]
 
-    # Create a shared thread pool executor with optimal worker count
-    with ThreadPoolExecutor(max_workers=max_workers) as executor:
-        # Process agents in batches
-        for i in range(0, len(agents), batch_size):
-            batch = agents[i : i + batch_size]
-            batch_results = loop.run_until_complete(
-                run_agents_concurrently_async(batch, task, executor)
-            )
-            results.extend(batch_results)
+        # Wait for all futures to complete and get results
+        for future in concurrent.futures.as_completed(futures):
+            try:
+                result = future.result()
+                results.append(result)
+            except Exception as e:
+                # Append the error if an agent fails
+                results.append(e)
 
     return results
 
@@ -415,141 +414,65 @@ def run_agents_with_tasks_concurrently(
     )
 
 
-# from joblib import Parallel, delayed
+def get_swarms_info(swarms: List[Callable]) -> str:
+    """
+    Fetches and formats information about all available swarms in the system.
+
+    Returns:
+        str: A formatted string containing names and descriptions of all swarms.
+    """
+    if not swarms:
+        return "No swarms currently available in the system."
+
+    swarm_info = [
+        "Available Swarms:",
+        "",
+    ]  # Empty string for line spacing
+
+    for idx, swarm in enumerate(swarms, 1):
+        swarm_info.extend(
+            [
+                f"[Swarm {idx}]",
+                f"Name: {swarm.name}",
+                f"Description: {swarm.description}",
+                f"Length of Agents: {len(swarm.agents)}",
+                f"Swarm Type: {swarm.swarm_type}",
+                "",  # Empty string for line spacing between swarms
+            ]
+        )
+
+    return "\n".join(swarm_info).strip()
 
 
-# def run_agents_joblib(
-#     agents: List[Any],
-#     tasks: List[str] = [],
-#     img: List[str] = None,
-#     max_workers: int = None,
-#     max_loops: int = 1,
-#     prefer: str = "threads",
-# ) -> List[Any]:
-#     """
-#     Executes a list of agents with their corresponding tasks concurrently using joblib.
+def get_agents_info(
+    agents: List[Union[Agent, Callable]], team_name: str = None
+) -> str:
+    """
+    Fetches and formats information about all available agents in the system.
 
-#     Each agent is expected to have a .run() method that accepts at least:
-#         - task: A string indicating the task to execute.
-#         - img: (Optional) A string representing image input.
+    Returns:
+        str: A formatted string containing names and descriptions of all swarms.
+    """
+    if not agents:
+        return "No agents currently available in the system."
 
-#     Args:
-#         agents (List[Any]): A list of agent instances.
-#         tasks (List[str], optional): A list of task strings. If provided, each agent gets a task.
-#                                      If fewer tasks than agents, the first task is reused.
-#         img (List[str], optional): A list of image strings. If provided, each agent gets an image.
-#                                    If fewer images than agents, the first image is reused.
-#         max_workers (int, optional): The maximum number of processes to use.
-#                                      Defaults to all available CPU cores.
-#         max_loops (int, optional): Number of times to execute the whole batch.
+    agent_info = [
+        f"Available Agents for Team: {team_name}",
+        "",
+    ]  # Empty string for line spacing
 
-#     Returns:
-#         List[Any]: The list of results returned by each agent’s run() method.
-#     """
-#     max_workers = max_workers or os.cpu_count()
-#     results = []
+    for idx, agent in enumerate(agents, 1):
+        agent_info.extend(
+            [
+                "\n",
+                f"[Agent {idx}]",
+                f"Name: {agent.agent_name}",
+                f"Description: {agent.agent_description}",
+                f"Role: {agent.role}",
+                f"Model: {agent.model_name}",
+                f"Max Loops: {agent.max_loops}",
+                "\n",
+            ]
+        )
 
-#     for _ in range(max_loops):
-#         results.extend(
-#             Parallel(n_jobs=max_workers, prefer=prefer)(
-#                 delayed(lambda a, t, i: a.run(task=t, img=i))(
-#                     agent,
-#                     (
-#                         tasks[idx]
-#                         if tasks and idx < len(tasks)
-#                         else (tasks[0] if tasks else "")
-#                     ),
-#                     (
-#                         img[idx]
-#                         if img and idx < len(img)
-#                         else (img[0] if img else None)
-#                     ),
-#                 )
-#                 for idx, agent in enumerate(agents)
-#             )
-#         )
-
-#     return results
-
-
-# # Example usage:
-# if __name__ == '__main__':
-#     # Dummy Agent class for demonstration.
-#     class Agent:
-#         def __init__(self, agent_name, max_loops, model_name):
-#             self.agent_name = agent_name
-#             self.max_loops = max_loops
-#             self.model_name = model_name
-
-#         def run(self, task: str, img: str = None) -> str:
-#             img_info = f" with image '{img}'" if img else ""
-#             return (f"{self.agent_name} using model '{self.model_name}' processed task: '{task}'{img_info}")
-
-#     # Create a few Agent instances.
-#     agents = [
-#         Agent(
-#             agent_name=f"Financial-Analysis-Agent_parallel_swarm{i}",
-#             max_loops=1,
-#             model_name="gpt-4o-mini",
-#         )
-#         for i in range(3)
-#     ]
-
-#     task = "How can I establish a ROTH IRA to buy stocks and get a tax break? What are the criteria"
-#     outputs = run_agents_process_pool(agents, tasks=[task])
-
-#     for i, output in enumerate(outputs):
-#         print(f"Output from agent {i+1}:\n{output}")
-
-# # Example usage:
-# if __name__ == '__main__':
-#     # A sample agent class with a run method.
-#     class SampleAgent:
-#         def __init__(self, name):
-#             self.name = name
-
-#         def run(self, task, device, device_id, no_clusterops):
-#             # Simulate some processing.
-#             return (f"Agent {self.name} processed task '{task}' on {device} "
-#                     f"(device_id={device_id}), no_clusterops={no_clusterops}")
-
-#     # Create a list of sample agents.
-#     agents = [SampleAgent(f"Agent_{i}") for i in range(5)]
-#     # Define tasks; if fewer tasks than agents, the first task will be reused.
-#     tasks = ["task1", "task2", "task3"]
-
-#     outputs = run_agents_with_tasks_concurrently(
-#         agents=agents,
-#         tasks=tasks,
-#         max_workers=4,
-#         device="cpu",
-#         device_id=1,
-#         all_cores=True,
-#         no_clusterops=False
-#     )
-
-#     for output in outputs:
-#         print(output)
-
-
-# # Example usage:
-# if __name__ == "__main__":
-#     # Initialize your agents (for example, 3 agents)
-#     agents = [
-#         Agent(
-#             agent_name=f"Financial-Analysis-Agent_parallel_swarm{i}",
-#             max_loops=1,
-#             model_name="gpt-4o-mini",
-#         )
-#         for i in range(3)
-#     ]
-
-#     # Generate a list of tasks.
-#     tasks = [
-#         "How can I establish a ROTH IRA to buy stocks and get a tax break?",
-#         "What are the criteria for establishing a ROTH IRA?",
-#         "What are the tax benefits of a ROTH IRA?",
-#         "How to buy stocks using a ROTH IRA?",
-#         "What are the limitations of a ROTH IRA?",
-#     ]
-#     outputs = run_agents_joblib(agents, tasks)
+    return "\n".join(agent_info).strip()
